# earwig
# Copyright (C) 2018  Frederic Peters
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.urlresolvers import reverse_lazy
from django.contrib import messages
from django.db.models import Q
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView, CreateView, DetailView, DeleteView

from ..sounds.models import Channel, Sound, STATUSES


class Homepage(ListView):
    model = Sound
    paginate_by = 10
    template_name = 'earwig/manager_home.html'

    def get_queryset(self, **kwargs):
        qs = self.model.objects.filter(status=self.get_status_filter()).select_related()
        query = self.get_query_filter()
        if query:
            qs = qs.filter(Q(title__icontains=query) |
                           Q(channel__title__icontains=query))
        return qs

    def get_query_filter(self):
        return self.request.GET.get('q') or ''

    def get_status_filter(self):
        return self.request.GET.get('q_status') or ''

    def get_context_data(self, **kwargs):
        context = super(Homepage, self).get_context_data(**kwargs)
        context['statuses'] = STATUSES
        return context

homepage = Homepage.as_view()


class Channels(ListView):
    model = Channel
    paginate_by = 10
    template_name = 'earwig/manager_channels.html'

channels = Channels.as_view()


class ChannelAdd(CreateView):
    model = Channel
    fields = ('title', 'channel_url', 'feed_url', 'policy')
    template_name = 'earwig/manager_channel_add.html'
    success_url = reverse_lazy('earwig-manager-homepage')

channel_add = ChannelAdd.as_view()


class ChannelDeleteView(DeleteView):
    model = Channel
    template_name = 'earwig/manager_channel_confirm_delete.html'
    success_url = reverse_lazy('earwig-channels')

channel_delete = ChannelDeleteView.as_view()


class SoundPublishView(DetailView):
    model = Sound

    def get(self, *args, **kwargs):
        self.get_object().publish()
        messages.success(self.request, _('%s has been published.') % self.get_object())
        return redirect('earwig-manager-homepage')

sound_publish = SoundPublishView.as_view()


class SoundRejectView(DetailView):
    model = Sound

    def get(self, *args, **kwargs):
        self.get_object().reject()
        messages.warning(self.request, _('%s has been rejected.') % self.get_object())
        return redirect('earwig-manager-homepage')

sound_reject = SoundRejectView.as_view()
